//------------------------------------------------------
// Redraw.c
// Redraw callback for "euler" plot.
// David Vainapel
// vainape@ibm.net
//------------------------------------------------------
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"
#include "eplot.h"

// global definition
PhRect_t rect;
int numOfObjects = 0;
float mx,my;
char *img = 0;

#define WinX( x ) ( ( x - XMIN ) * mx ) + MARGINX
#define WinY( y ) ( ( y - YMIN ) * my ) + MARGINY

static PgColor_t pColor[] =
{
	Pg_WHITE,
	Pg_BLACK,
	PgRGB( 0, 168, 0 ),
	PgRGB( 0, 0, 168 ),
	PgRGB( 0, 0, 256 ),
	PgRGB( 168, 0, 0 ),
	PgRGB( 0, 168, 168 ),
	PgRGB( 168, 0, 168 ),
	PgRGB( 168, 84, 0 ),
	PgRGB( 84, 255, 84 ),
	PgRGB( 84, 255, 255 ),
	PgRGB( 255, 84, 84 ),
	PgRGB( 255, 84, 255 ),
	PgRGB( 255, 255, 84 ),
	PgRGB( 84, 84, 255 ),
	PgRGB( 168, 168, 168 ),
	PgRGB( 84, 84, 84 ),
};

//--------------------------------------------- 
// This is not like in Grothmann program. 
// Let be 0 for white, 1, 2, 3 for r, g, b
//----------------------------------------------
static void SetHue( float hue , int color )
{
	int r = 255, g = 255, b = 255;
	switch( color )
	{
	case 0:
		r = 0;	
		break;
	case 1:
		g = 0;	
		break;
	case 2:
		b = 0;	
		break;
	}
	PgSetFillColor( PgRGB( (int)(r * hue), (int)(b * hue), (int)(g *hue)) );
	PgSetStrokeColor( pColor[1] );
}

static void DrawLine( Gline *msg)
{
	int x, y, x1, y1;
	x = WinX( msg->c );
	y = WinY( msg->r );
	x1 = WinX( msg->c1 );
	y1 = WinY( msg->r1 );
	PgSetStrokeColor( pColor[ msg->col ] );
	PgDrawILine( x, y, x1, y1 );
}

static void DrawText( Gtext *msg)
{
	static int flag[3] = { Pg_TEXT_LEFT, Pg_TEXT_CENTER, Pg_TEXT_RIGHT }; 
	PhPoint_t p;
	p.x = WinX( msg->c );
	p.y = WinY( msg->r - 5);
	PgSetTextColor( pColor[ msg->color ] );
	PgDrawText( msg->text, strlen(msg->text), &p, flag[msg->alignment] );
}

//--------------------------------------------- 
// This is not like in Grothmann program. 
// Not all markers and not in orignal order.
//----------------------------------------------
static void DrawMarker( Gmarker *msg)
{
	PhPoint_t radii={3,3}, center;
	PgSetStrokeColor( pColor[ msg->color ] );
	center.x = WinX( msg->c );
	center.y = WinY( msg->r );

	switch( msg->type )
	{
	case 0:
		PgDrawILine( center.x-3, center.y-3, center.x+3, center.y+3 );
		PgDrawILine( center.x+3, center.y-3, center.x-3, center.y+3 );
		break;
	case 1:
		PgDrawEllipse( &center, &radii, Pg_DRAW_STROKE);
		break;
	case 2:
		PgDrawILine( center.x-3, center.y, center.x+3, center.y );
		PgDrawILine( center.x, center.y-3, center.x, center.y+3 );
		break;
	case 3:
		PgDrawILine( center.x-3, center.y, center.x+3, center.y );
		PgDrawILine( center.x, center.y-3, center.x, center.y+3 );
		break;
	default:
		PgDrawIRect( center.x-3, center.y-3, center.x+3, center.y+3,
			Pg_DRAW_STROKE );
		break;
	}
}

// 
// <======= MORE WORK
//
void DrawFill( Gfill *msg )
{
	PhPoint_t *p;
	PhPoint_t origin = {0,0};
	int i;
	p = (PhPoint_t*) malloc( msg->n * sizeof(PhPoint_t) );
	for( i = 0; i < msg->n; i++ )
	{
		p[i].x = WinX( msg->p[i * 2] );
		p[i].y = WinY( msg->p[i * 2+1] );
	}
	SetHue( msg->hue, msg->color );
	PgDrawPolygon( p, msg->n, &origin, Pg_DRAW_FILL_STROKE | Pg_CLOSED );
	free( p );
}

void DrawBar( Gbar *msg)
{
	int x,y,x1,y1;
	x = WinX( msg->c );
	y = WinY( msg->r );
	x1 = WinX( msg->c1 );
	y1 = WinY( msg->r1 );
	SetHue( msg->hue, msg->color );
	if( msg->border )
	{
		PgSetStrokeColor( pColor[1] );
		PgDrawIRect( x, y, x1, y1, Pg_DRAW_FILL_STROKE );
	}
	else
	{
		PgDrawIRect( x, y, x1, y1, Pg_DRAW_FILL );
	}
}

static void DrawObjects()
{
	int i;
	char *ptr = img;
	for( i = 0; i < numOfObjects; i++ )
	{
		Msg *msg = (Msg*) ptr;		
		switch( msg->code )
		{
		case LINE:
			DrawLine( (Gline*)ptr ); 
			break;
		case TEXT:
			DrawText( (Gtext*)ptr ); 
			break;
		case MARKER:
			DrawMarker( (Gmarker*)ptr ); 
			break;
		case FILL:
			DrawFill( (Gfill*)ptr ); 
			break;
		case BAR:
			DrawBar( (Gbar*)ptr ); 
			break;
		}
		ptr += msg->size;
	}
}

void Redraw( PtWidget_t *widget, PhTile_t *damage )  {
	PtSuperClassDraw( PtBasic, widget, damage );
	PtBasicWidgetCanvas( widget, &rect );
	mx = (rect.lr.x - 2 * MARGINX  ) / (XMAX - XMIN );
	my = (rect.lr.y  - 2 * MARGINY  )/ (YMAX - YMIN ) ;
	PtClipAdd( widget, &rect );
	PgSetFont( "helv12b" );		
	PgSetStrokeColor( pColor[0] );
	PgSetFillColor( pColor[0] );			// clear the screen
	PgDrawRect( &rect, Pg_DRAW_FILL|Pg_DRAW_STROKE);
	DrawObjects();
	PtClipRemove();
}

