//------------------------------------------------------
// TimePlot.c
// Timer callback for "euler" plot.
// David Vainapel
// vainape@ibm.net
//------------------------------------------------------
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"
#include "eplot.h"

extern int waitForMouse;
extern PhRect_t rect;
extern int numOfObjects;
extern char *img;


static int imgPtr = 0;

#define BLOCK 0x10000

//------------------------------------------
//------------------------------------------
static void SaveObject( char *msg, int size )
{
	static int imgSize = 0;
	char *newbuf;
	if( imgSize - imgPtr < size )
	{
		newbuf = realloc( img, imgSize + BLOCK );	// add some ram
		if( newbuf == 0 )
			return;		// no more ram, but do nothing.
		img = newbuf;
		imgSize += BLOCK; 
	}
	memcpy( &img[imgPtr], msg, size );
	imgPtr += size;
	numOfObjects ++;
}

static void Clear()
{
	imgPtr = 0;	
	numOfObjects = 0;
}

//-----------------------------------------------------------
// Check input every 300 msec
// Input expected:
//	 Request for mouse position.
//   Clear screen
//	 Draw object
//-----------------------------------------------------------
int
TimePlot( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	static char msgBuf[10000];			// <======== More work
	static int msgPtr = 0;
	static int msgSize;

	int redraw = 0;			// indicates if needs redraw 
	while( 1 )
	{
		int count;
		Msg *msg = (Msg*)msgBuf;
		if( msgPtr == 0)		// if msgPtr = 0, read message size
			msgSize = sizeof(long);
		count = read( STDIN_FILENO, &msgBuf[ msgPtr], msgSize - msgPtr );
		if(count > 0)
			msgPtr += count;

		// if no more data, then redraw and return
		if( count < msgSize - msgPtr)
		{
			if( redraw )
				PtDamageExtent( ABW_Graph, &rect );
				return Pt_CONTINUE; 						// next time
		}
		if( msgPtr == sizeof(long) )
		{
			msgSize = msg->size;		// get rest of message
			continue;
		}
		redraw = 1;						// something to do

		switch( msg->code )
		{
		case POSITION:					// wait for mouse click
			waitForMouse = 1;
			break;

		case CLEAR:						// clear window
			Clear();
			break;
		
		default:						// all others are graphic objects
			SaveObject( msgBuf, msg->size );
			break;
		}
		msgPtr = 0;						// clear msg buffer
	}
}

